@@ -8,7 +8,7 @@ EM::VERSION >= '0.12.10' or abort 'eventmachine 0.12.10 is required'
 # a single-threaded app dispatch.  It is suited for slow clients,
 # and can work with slow applications via asynchronous libraries such as
 # {async_sinatra}[http://github.com/raggi/async_sinatra],
-# {Cramp}[http://m.onkey.org/2010/1/7/introducing-cramp],
+# {Cramp}[http://cramp.in/],
 # and {rack-fiber_pool}[http://github.com/mperham/rack-fiber_pool].
 #
 # It does not require your Rack application to be thread-safe,
@@ -23,7 +23,7 @@ EM::VERSION >= '0.12.10' or abort 'eventmachine 0.12.10 is required'
 # {async_sinatra}[http://github.com/raggi/async_sinatra].
 #
 # For a complete asynchronous framework,
-# {Cramp}[http://m.onkey.org/2010/1/7/introducing-cramp] is fully
+# {Cramp}[http://cramp.in/] is fully
 # supported when using this concurrency model.
 #
 # This model is fully-compatible with
@@ -50,6 +50,21 @@ module Rainbows::EventMachine
 
   include Rainbows::Base
 
+  # Cramp (and possibly others) can subclass Rainbows::EventMachine::Client
+  # and provide the :em_client_class option.  We /don't/ want to load
+  # Rainbows::EventMachine::Client in the master process since we need
+  # reloadability.
+  def em_client_class
+    case klass = Rainbows::O[:em_client_class]
+    when Proc
+      klass.call # e.g.: proc { Cramp::WebSocket::Rainbows }
+    when Symbol, String
+      eval(klass.to_s) # Object.const_get won't resolve multi-level paths
+    else # @use should be either :EventMachine or :NeverBlock
+      Rainbows.const_get(@use).const_get(:Client)
+    end
+  end
+
   # runs inside each forked worker, this sits around and waits
   # for connections and doesn't die until the parent dies (or is
   # given a INT, QUIT, or TERM signal)
@@ -63,11 +78,11 @@ module Rainbows::EventMachine
     EM.epoll
     EM.kqueue
     logger.info "#@use: epoll=#{EM.epoll?} kqueue=#{EM.kqueue?}"
-    client_class = Rainbows.const_get(@use).const_get(:Client)
+    client_class = em_client_class
     max = worker_connections + LISTENERS.size
     Rainbows::EventMachine::Server.const_set(:MAX, max)
     Rainbows::EventMachine::Server.const_set(:CL, client_class)
-    client_class.const_set(:APP, Rainbows.server.app)
+    Rainbows::EventMachine::Client.const_set(:APP, Rainbows.server.app)
     EM.run {
       conns = EM.instance_variable_get(:@conns) or
         raise RuntimeError, "EM @conns instance variable not accessible!"
@@ -84,6 +99,7 @@ module Rainbows::EventMachine
         end
       end
     }
+    EM.reactor_thread.join if EM.reactor_running?
   end
 end
 # :enddoc:
